// XDemo18: Reads I-7000 series module's ID.

// Compiler: BC++ 3.1
// Compile mode: large
// Project: user.c
//			v7000.c
//			vModbus.c			[after XS8_3002]
//          ..\Lib\8000E.Lib
//          ..\Lib\TCPIPL.Lib
//          ..\Lib\XS8_NNNN.Lib, with NNNN being the lib file's version.

//   19~!@#$  -> Any non-null command will be accepted.

//   This demo shows how to communication with 7000 series which connected
//   to COM3 of 8000E.

//   Hardware: 8000E + any 7000 series module whose address is 01,
//                                                  baudrate is 9600,
//                                                  checksum is disable.

// Refer 8000\843x883x\TCP\Doc\[Big5|Eng|Gb2312]\Vxcomm.htm 
//       8000\843x883x\TCP\Xserver\Xserver.htm
//       8000\843x883x\TCP\Xserver\Function.htm   
// to get more information.

// [25/Oct/2002] by Kevin

#include<stdio.h>
#include<string.h>
#include "..\lib\module.h"


char cStr[9];       // Receive data from 7000: "!017021" + 0x0c
char cModuleID[5];  // Store module ID: "7021" + 0x00

void UserCount(void)
{
// user's timer trigger function

	int i;

	SendCmdTo7000(3,"$01M",0);
	ReceiveResponseFrom7000(3,cStr,5000,0);
	
	// cut "!01" and cr
	strcpy(cModuleID,cStr+3);
	cModuleID[4]=0;

	// Show moduleID to PC monitor
	printCom(1,"Address: 01,  ModuleID:%s\n\r",cModuleID);
}
void UserEnd(void)
{
	Print("Call UserEnd().\r\n");
	StopUserTimerFun();
}
void UserInit(void)
{
/*
	In this function, user CAN:
	1. initialize user's program.
	2. set time interval for calling UserCount().
	3. set initial value of I/O or variables for UserLoopFun().
	4. set initial value of I/O or variables for another functions.
	5. change the default TCP PORT 10000/9999/502 to others value. [after XS8_3004.lib]
	   Syntax:
    	Port10000=newport_10000;  for calling UserCmd       (user.c)
    	Port9999=newport_9999;    for calling VcomCmd7000   (v7000.c)
    	Port502=newport_502;      for calling VcomCmdModbus (vModbus.c) [after XS8_3002.lib]
	    PortUser=newport_User;    for calling VcomCmdUser   (user.c)    [after XS8_3005.lib]
	    
		Default port value:
		Port10000=10000;
		Port9999=9999;
		Port502=502;
		PortUser=0;
		If the port value is 0, Xserver will not 
		listen that port. That means the port will 
		be disable.
		
	Please refer to XDemo09 & XDemo11 for example code
*/	

	InitLib();	// To use [MiniOS7 for 8000 new], you need call this function
				// at begin of c program. 
				// XS8_nnnn.Lib has not add this function to it's kernel.
				// So, you need call that in UserInit. 
				// To know what difference between original MiniOS7 and new MiniOS7,
				// please refer CD:\Napdos\MiniOS7\8000new\8000-new_eng.txt
				//                                         8000-new_big5.txt
				//                                         8000-new_gb2312.txt
				// [26,Oct,2002] by Kevin
					
	AddUserTimerFunction(UserCount,1000);
	SetBaudrate(1,115200L);
    DisableCom(3);
}

void UserLoopFun(void)
{
	// VxComm.exe will call this function every scan time
	// refer to demo11 for scan-time evaluation
}


int UserCmd(unsigned char *Cmd,unsigned char *Response)
{
	// user's command interpreter
	// refer to all demo
	char cStr[20];
	if(Cmd[0])
	{
		sprintf(cStr,"ModuleID:%s",cModuleID);
		strcpy(Response,cStr);
	}
	return 1;               // return OK
}

int VcomUserBinaryCmd(TCPREADDATA *p)
{
 /* VXCOMM.EXE 2.6.12(09/04/2001) or later will support this function.

		TCP PORT 10000, command 23 will call this function.
		user can get the following message:
		p->ReadUartChar : the buffer store the command data(include "23")
		p->Length : the command data length(include the two byte "23")
		p->Socket : the socket number that receives the command, that is, when the user function wants
					to return a message to the client, just use the socket to send data.
					use:  VcomSendSocket(p->Socket,pdata,datalength);
 */
	VcomSendSocket(p->Socket,"User-defined command(23)",24);	// return 24 bytes.
 	return 1;	/* any value will be accept */
}


int VcomCmdUser(TCPREADDATA *p)  
{ 
	/*
    VCOM3005 (Feb,22,2002) or later will call this function for PortUser.
    
    When packets received by TCP PORT PortUser(user defined) 
    of 7188E/8000E, Xserver will call this function.
    user can get the following message:
    p->ReadUartChar : the buffer store the command data.
                      Maximum length of p->ReadUartChar is 32767 bytes.
    p->Length : the command data length.
    p->Socket : the socket number that receive the command, that is when the user function wants 
                return message to the client, just use the socket to send data.
                usage:  VcomSendSocket(p->Socket,pdata,datalength);
	*/
  
	/* here just send back the command to the client. */
	VcomSendSocket(p->Socket,p->ReadUartChar,p->Length);
 	return 1;	/* any value will be accept */
}

void PortUserStart(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    	When a TCP/IP client connects to the 7188E/8000E via the user's defined port(PortUser), 
	    the Xserver calls the function once.
    
    	You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}

void Port9999Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    
    	When a TCP/IP client connects to the 7188E/8000E TCP port 9999, 
	    the Xserver calls the function once.
    
		You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    	
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}
 
void Port502Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
    	or later version supports this function.
    
	    When a TCP/IP client connects to the 7188E/8000E TCP port 502, 
    	the Xserver calls the function once.
    
	    You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
	    skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}