// XDemo24: Uses countdown timer

// Compiler: BC++ 3.1
// Compile mode: large
// Project: user.c
//			v7000.c
//			vModbus.c			[after XS8_3002]
//          ..\LIB\8000E.Lib
//          ..\LIB\TCPIPL.Lib
//          ..\LIB\XS8_NNNN.Lib, with NNNN being the lib file's version.

//   When 7188E power on, the demo will start running countdown function.

//     There are 8 countdown timer (channel 0 to channel 7)
//     This demo uses channel 0.
//     The countdown timer initial value is 1000 ms.
//     When the countdown timer value become 0,
//     the value of LED will increase.

//   Hardware: 8000E

// Refer 8000\843x883x\TCP\Doc\[Big5|Eng|Gb2312]\Vxcomm.htm 
//       8000\843x883x\TCP\Xserver\Xserver.htm
//       8000\843x883x\TCP\Xserver\Function.htm   
// to get more information.

// [30/Nov/2001] by Kevin

#include <string.h>
#include <conio.h>
#include <stdio.h>
#include "..\lib\module.h"

extern int bLedControl;

int iNumber;
unsigned long ulCountValue;
char cLED[6];

void UserCount(void)
{
// user's timer trigger function
}
void UserEnd(void)
{
	Print("Call UserEnd().\r\n");
	StopUserTimerFun();
}
void UserInit(void)
{
/*
	In this function, user CAN:
	1. initialize user's program.
	2. set time interval for calling UserCount().
	3. set initial value of I/O or variables for UserLoopFun().
	4. set initial value of I/O or variables for another functions.
	5. change the default TCP PORT 10000/9999/502 to others value. [after XS8_3004.lib]
	   Syntax:
    	Port10000=newport_10000;  for calling UserCmd       (user.c)
    	Port9999=newport_9999;    for calling VcomCmd7000   (v7000.c)
    	Port502=newport_502;      for calling VcomCmdModbus (vModbus.c) [after XS8_3002.lib]
	    PortUser=newport_User;    for calling VcomCmdUser   (user.c)    [after XS8_3005.lib]
	    
		Default port value:
		Port10000=10000;
		Port9999=9999;
		Port502=502;
		PortUser=0;
		If the port value is 0, Xserver will not 
		listen that port. That means the port will 
		be disable.
		
	Please refer to XDemo09 & XDemo11 for example code
*/	

	InitLib();	// To use [MiniOS7 for 8000 new], you need call this function
				// at begin of c program. 
				// XS8_nnnn.Lib has not add this function to it's kernel.
				// So, you need call that in UserInit. 
				// To know what difference between original MiniOS7 and new MiniOS7,
				// please refer CD:\Napdos\MiniOS7\8000new\8000-new_eng.txt
				//                                         8000-new_big5.txt
				//                                         8000-new_gb2312.txt
				// [26,Oct,2002] by Kevin
				
	Init5DigitLed();
	bLedControl=0;
	TimerOpen();
	CountDownTimerStart(0,1000); // use channel 0, Count Value is 1000 ms
}

void UserLoopFun(void)
{
// VxComm.exe will call this function every scan time
	int i;

	CountDownTimerReadValue(0,&ulCountValue);
	
	if (ulCountValue==0)
	{
		CountDownTimerStart(0,1000);
		
		iNumber++;
		sprintf(cLED,"%5d",iNumber);
		for (i=1;i<=5;i++)
			Show5DigitLed(i,cLED[i-1]-'0');
	}
}


int UserCmd(unsigned char *Cmd,unsigned char *Response)
{
// user's command interpreter
	strcpy(Response,Cmd);
	return 1;               // return OK
}

int VcomUserBinaryCmd(TCPREADDATA *p)
{
 /* VXCOMM.EXE 2.6.12(09/04/2001) or later will support this function.

		TCP PORT 10000, command 23 will call this function.
		user can get the following message:
		p->ReadUartChar : the buffer store the command data(include "23")
		p->Length : the command data length(include the two byte "23")
		p->Socket : the socket number that receives the command, that is, when the user function wants
					to return a message to the client, just use the socket to send data.
					use:  VcomSendSocket(p->Socket,pdata,datalength);
 */
	VcomSendSocket(p->Socket,"User-defined command(23)",24);	// return 24 bytes.
  	return 1;	/* any value will be accept */
}


int VcomCmdUser(TCPREADDATA *p)  
{ 
	/*
    VCOM3005 (Feb,22,2002) or later will call this function for PortUser.
    
    When packets received by TCP PORT PortUser(user defined) 
    of 7188E/8000E, Xserver will call this function.
    user can get the following message:
    p->ReadUartChar : the buffer store the command data.
                      Maximum length of p->ReadUartChar is 32767 bytes.
    p->Length : the command data length.
    p->Socket : the socket number that receive the command, that is when the user function wants 
                return message to the client, just use the socket to send data.
                usage:  VcomSendSocket(p->Socket,pdata,datalength);
	*/
  
	/* here just send back the command to the client. */
	VcomSendSocket(p->Socket,p->ReadUartChar,p->Length);
 	return 1;	/* any value will be accept */
}

void PortUserStart(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    	When a TCP/IP client connects to the 7188E/8000E via the user's defined port(PortUser), 
	    the Xserver calls the function once.
    
    	You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}

void Port9999Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    
    	When a TCP/IP client connects to the 7188E/8000E TCP port 9999, 
	    the Xserver calls the function once.
    
		You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    	
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}
 
void Port502Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
    	or later version supports this function.
    
	    When a TCP/IP client connects to the 7188E/8000E TCP port 502, 
    	the Xserver calls the function once.
    
	    You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
	    skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}