// XDemo58: The Client fuction read the analog input of the 7018 module via Internet

// Compiler: BC++ 3.1
// Compile mode: large
// Project: user.c
//			Vsocket.c            
//			v7000.c
//			vModbus.c			[after vcom3002.lib]
//          ..\Lib\8000L.Lib
//          ..\Lib\TCPIPL.Lib
//          ..\Lib\XS8_NNNN.Lib, with NNNN being the lib file's version.
//
//		Use the Switch Key of the 8000E module to control the connecting of the i7188E module
//
//		UP:Build the connecting between the i8431 and i7188E.
//
//		DOWN:Kill the connecting between the i8431 and i7188E. 
//
//   This demo provides the client functions using TCP/IP protocol for 8000E modules.Apply the functions to
//   read the 5 channels of the analog input of the 7018 module through the 7188E modules via Internet,
//   but doesn't need any the control of the PC.
//
//   Hardware: 8431,7188E modules * 1pcs 
//             7018D * 1  Address->01 
//             8000E(client/server) <--->Hub<--->7188E(server)(RS485)<--->7018D(RS485)
//     
//   Note:Chang the IP and Port number of the server module in the UserInit() function.
//		  The users can change the timeout to receive data. 
//		  Please refer ReceiveSocketBinaryCmd() in Vsocket.h for more detail information.
//	 
//         
// Refer 8000\843x883x\TCP\Doc\[Big5|Eng|Gb2312]\Vxcomm.htm 
//       8000\843x883x\TCP\Xserver\Xserver.htm
//       8000\843x883x\TCP\Xserver\Function.htm   
// to get more information.

// [14/July/2002] by Sean

#include<string.h>
#include<stdio.h>
#include "..\lib\module.h"
#include "Vsocket.h"

int iSocketflag; // the flag use to detect whether the socket is active;

char InBuf[1023],OutBuf[1023];  	
int iOutskt;
unsigned char iIpStr[17];
unsigned int iPort;
unsigned long lTimeTicks1;


void UserCount(void)
{

}
void UserEnd(void)
{
	Print("Call UserEnd().\r\n");
	StopUserTimerFun();
}
void UserInit(void)
{
/*
	In this function, user CAN:
	1. initialize user's program.
	2. set time interval for calling UserCount().
	3. set initial value of I/O or variables for UserLoopFun().
	4. set initial value of I/O or variables for another functions.
	5. change the default TCP PORT 10000/9999/502 to others value. [after XS8_3004.lib]
	   Syntax:
    	Port10000=newport_10000;  for calling UserCmd       (user.c)
    	Port9999=newport_9999;    for calling VcomCmd7000   (v7000.c)
    	Port502=newport_502;      for calling VcomCmdModbus (vModbus.c) [after XS8_3002.lib]
	    PortUser=newport_User;    for calling VcomCmdUser   (user.c)    [after XS8_3005.lib]
	    
		Default port value:
		Port10000=10000;
		Port9999=9999;
		Port502=502;
		PortUser=0;
		If the port value is 0, Xserver will not 
		listen that port. That means the port will 
		be disable.
		
	Please refer to XDemo09 & XDemo11 for example code
*/	

	InitLib();	// To use [MiniOS7 for 8000 new], you need call this function
				// at begin of c program. 
				// XS8_nnnn.Lib has not add this function to it's kernel.
				// So, you need call that in UserInit. 
				// To know what difference between original MiniOS7 and new MiniOS7,
				// please refer CD:\Napdos\MiniOS7\8000new\8000-new_eng.txt
				//                                         8000-new_big5.txt
				//                                         8000-new_gb2312.txt
				// [26,Oct,2002] by Kevin
					
	sprintf(iIpStr,"192.168.0.245");
	iPort=10002;	
	InstallCom1(115200L,8,0,1);
	iSocketflag=0;		        
	printCom1("ServerIP = %s  ,Port = %d \n\r",iIpStr,iPort);
}

void UserLoopFun(void)
{
	int iRet,iLoop;

	strnset(InBuf,0,sizeof(InBuf)); // clear the InBuf array	
	if(IsSystemKey())
	{
		switch(GetSystemKey())
		{
			case SKEY_UP:
				printCom1("Press UP key now\n\r");
				if(iSocketflag==0)
				{
					iOutskt=BuildSocket(iIpStr,iPort,1600);
					if (iOutskt==-1)
						printCom1("Connect error!!\n\r");
				}					
				iSocketflag=1;				
				break;
			case SKEY_DOWN:
				printCom1("Press DOWN key now\n\r");
				if(iSocketflag==1)
				{
					KillSocket(iOutskt);
					printCom1("InBuf=%s\n\r",InBuf);					
				}
				iSocketflag=0;
				break;
	
		}//of switch
		// Send data to PC
	}	

	if (iSocketflag==1)   //Systemkey "UP" trigger this functions while isocketflag==1 (constructed the socket)
	{
		for(iLoop=0;iLoop<5;iLoop++)
		{
			sprintf(OutBuf,"#01%d\r",iLoop);
			//sprintf(OutBuf,"01\r",iLoop);
			iRet=SendSocketBinaryCmd(iOutskt,OutBuf,strlen(OutBuf)); 

			if (iRet==-1)// write error
			{               
				printCom1("Echo Send error!!\n\r",InBuf);
				KillSocket(iOutskt);
				iSocketflag=0;
    		}	    
    		else	
		    	printCom1("size= %d  OutBuf= %s\n\r",iRet,OutBuf);

			iRet=ReceiveSocketBinaryCmd(iOutskt,InBuf,sizeof(InBuf),500);//Timeout large than 800ms,8000E will reboot by WDT unless use DiableWDT.
			if (iRet==-1)// read error
			{							
				printCom1("Echo Receive error\n\r",InBuf);
				KillSocket(iOutskt);
				iSocketflag=0;
	    	}
    		else
	    	{
		    	InBuf[iRet]=0;	// add zero to end
				printCom1("InBuf= %s\n\rsize= %d\n\r",InBuf,iRet);
				printCom1("Connect->send->receive->Close Total time=%d\r\n",*TimeTicks-lTimeTicks1);		    	
    		}
		}    	
    	Delay(700);//timeout can't large than 800ms otherwise 8000E will reboot by WDT
	}			
}

int UserCmd(unsigned char *Cmd,unsigned char *Response)
{
	// user's command interpreter
	// Maximum length of Cmd is 1458 bytes.
	// Maximum length of Response is 1024 bytes.
	// 19 192.168.254.20 string , Send "string" to IP 192.168.254.20 via TCP/IP socket
	strcpy(Response,Cmd);
	return 1;	/* return error */
}

int VcomUserBinaryCmd(TCPREADDATA *p)
{
/* VXCOMM.EXE 2.6.12(09/04/2001) or later will support this function.

	TCP PORT 10000, command 23 will call this function.
	user can get the following message:
	p->ReadUartChar : the buffer store the command data(include "23")
	                  Maximum length of p->ReadUartChar is 32767 bytes.
	p->Length : the command data length(include the two byte "23")
	p->Socket : the socket number that receives the command, that is, when the user function wants
		to return a message to the client, just use the socket to send data.
		usage:  VcomSendSocket(p->Socket,pdata,datalength);
*/
	VcomSendSocket(p->Socket,"User-defined command(23)",24);	// return 24 bytes.
	return 1;	/* any value will be accept */
}

int VcomCmdUser(TCPREADDATA *p)  
{ 
	/*
    VCOM3005 (Feb,22,2002) or later will call this function for PortUser.
    
    When packets received by TCP PORT PortUser(user defined) 
    of 7188E/8000E, Xserver will call this function.
    user can get the following message:
    p->ReadUartChar : the buffer store the command data.
                      Maximum length of p->ReadUartChar is 32767 bytes.
    p->Length : the command data length.
    p->Socket : the socket number that receive the command, that is when the user function wants 
                return message to the client, just use the socket to send data.
                usage:  VcomSendSocket(p->Socket,pdata,datalength);
	*/
  
	/* here just send back the command to the client. */
	VcomSendSocket(p->Socket,p->ReadUartChar,p->Length);
 	return 1;	/* any value will be accept */	
}

void PortUserStart(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    	When a TCP/IP client connects to the 7188E/8000E via the user's defined port(PortUser), 
	    the Xserver calls the function once.
    
    	You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}

void Port9999Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
	    or later version supports this function.
    
    	When a TCP/IP client connects to the 7188E/8000E TCP port 9999, 
	    the Xserver calls the function once.
    
		You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    	
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
    	skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}
 
void Port502Start(int skt)
{
    /* 
    	XS8_3200.Lib Version 3.2.00 (20,Apr,2004) 
    	or later version supports this function.
    
	    When a TCP/IP client connects to the 7188E/8000E TCP port 502, 
    	the Xserver calls the function once.
    
	    You can use function VcomSendSocket to send a message to the client 
    	when a connection is established.
    
    For example:
    	VcomSendSocket(skt,"Connection is established.",26); //return 26 bytes.
 
	    skt: socket number assigned to the TCP/IP client.
    */
    
    skt=skt; //do nothing
}